<?php
/*
 * Plugin Name: 3DPrint: Custom Price Mod
 * Author: Sergey Burkov
 * Description: Custom price formulas for 3DPrint
 * Text Domain: cpm
 * WC requires at least: 3.0
 * WC tested up to: 9.3.9
 * Version: 1.3.4
*/

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

include_once('legacy.php');

add_action( 'before_woocommerce_init', function() {
	if ( class_exists( \Automattic\WooCommerce\Utilities\FeaturesUtil::class ) ) {
		\Automattic\WooCommerce\Utilities\FeaturesUtil::declare_compatibility( 'custom_order_tables', __FILE__, true );
	}
} );


add_action( 'wp_enqueue_scripts', 'cpm_enqueue_scripts_frontend' );


function cpm_enqueue_scripts_frontend() {
	global $post, $woocommerce;
	#include_once( ABSPATH . 'wp-admin/includes/plugin.php' );
	if (!function_exists('get_woocommerce_currency_symbol')) {
		return;
	}

	if( !function_exists( 'p3d_activate' ) ) {
		return;
	}

	wp_enqueue_script( 'cpm.js',  plugin_dir_url( __FILE__ ).'js/cpm.js', array( '3dprint-frontend.js' ), '1.3.4' );
}



add_filter('3dprint_calculate_printing_cost', 'cpm_calculate_printing_cost', 10 ,6);
function cpm_calculate_printing_cost( $total, $printer_id, $material_id, $coating_id, $product_info, $attributes ) {

	$printing_cost=$material_cost=$coating_cost=0;
	$materials_array = p3d_get_option( '3dp_materials' );
	$material = $materials_array[$material_id];
	$material_coeff = 100;
	$p3d_attr_prices=p3d_get_option('3dp_attr_prices');
	$settings = p3d_get_option('3dp_settings');

	$product = new WC_Product_Variable( $product_info['product_id'] );
	$min_price = $product->get_price();
	$currency_rate = p3d_get_currency_rate();
	if ($currency_rate>1) {
		$min_price/=$currency_rate;
	}

	$p3d_file_url = get_post_meta($product_info['product_id'], 'p3d_file_url', true); 
	$p3d_product_price_type = get_post_meta($product_info['product_id'], 'p3d_product_price_type', true); 
	if (isset($attributes['attribute_pa_p3d_infill'])) {
		$infill_value = (float)$attributes['attribute_pa_p3d_infill'];
	}
	else {
		$infill_value = 0;
	}
	
	$infill_id = (int)$attributes['p3d_infill_id'];
	$infills_array = p3d_get_option( '3dp_infills' );
	$infill=$infills_array[$infill_id];
	$postprocessings_array = p3d_get_option( '3dp_postprocessings' );
	if (isset($attributes['p3d_postprocessing_ids']) && count($attributes['p3d_postprocessing_ids'])) {
		$postprocessings_ids = (array)$attributes['p3d_postprocessing_ids'];
	}
	else {
		$postprocessings_ids = array();
	}
	$postprocessing_price_fields = array('', '1');

	$printer_price_fields = array('', '1', '2', '3', '4');
	$material_price_fields = array('', '1', '2');
	$coating_price_fields = array('', '1');
	$infill_price_fields = array('', '1');

	$printing_cost = 0;
	$material_cost = 0;
	$coating_cost = 0;
	$infill_cost = 0;
	$postprocessing_cost = 0;

	$printing_pct = 0;
	$material_pct = 0;
	$coating_pct = 0;
	$infill_pct = 0;
	$postprocessing_pct = 0;


	if (count($attributes)) {
		foreach ($attributes as $attr_name => $attr_value) {
			$attr_name=str_replace('attribute_', '', $attr_name);
			if (isset($p3d_attr_prices[$attr_name][$attr_value])) {
				$attr_price=(float)$p3d_attr_prices[$attr_name][$attr_value]['price'];
				$attr_price_type=$p3d_attr_prices[$attr_name][$attr_value]['price_type'];
				$attr_pct_type=$p3d_attr_prices[$attr_name][$attr_value]['pct_type'];
				if ($attr_price_type=='pct') {
					if ($attr_pct_type=='material_amount') {
						$material_coeff+=$attr_price;

					}

				}
			}
		}
	}



	if (!empty($coating_id)) {
		$coatings_array = p3d_get_option( '3dp_coatings' );
		$coating = $coatings_array[$coating_id];
	}
	
	$printers_array = p3d_get_option( '3dp_printers' );
	$printer = $printers_array[$printer_id];


	$printing_volume = $product_info['model']['material_volume']*($material_coeff/100);
	$weight = $material['density']*$product_info['model']['material_volume']*($material_coeff/100);
	$removed_material_volume = $product_info['model']['box_volume'] - $printing_volume;

	if ($printer['type']=='dlp') {
		if ( version_compare( P3D_VERSION, '3.8.2.5', '<=' ) ) {
			$product_info['print_time'] = p3d_calculate_dlp_print_time_legacy($printer, $product_info['model']);
		}
		else {
			$product_info['print_time'] = p3d_calculate_dlp_print_time($printer, $material, $product_info['model']);
		}
	}

	for ($p=0;$p<count($material_price_fields);$p++) {

		if ( is_numeric( $material['price'.$material_price_fields[$p]] ) ) {
			if ( $material['price_type'.$material_price_fields[$p]]=='cm3' ) {
				$material_cost += ( $printing_volume ) * $material['price'.$material_price_fields[$p]];
			}
			elseif ( $material['price_type'.$material_price_fields[$p]]=='cm2' ) {
				$material_cost += $product_info['model']['surface_area'] * $material['price'.$material_price_fields[$p]];
			}
			elseif( $material['price_type'.$material_price_fields[$p]]=='box_volume' ) {
				$material_cost += $product_info['model']['box_volume'] * $material['price'.$material_price_fields[$p]];
			}
			elseif ( $material['price_type'.$material_price_fields[$p]]=='box_xy' ) {
				$material_cost += ($product_info['model']['x_dim'] * $product_info['model']['y_dim']) * $material['price'.$material_price_fields[$p]];
			}
			elseif ( $material['price_type'.$material_price_fields[$p]]=='support_material_volume' ) {
				$material_cost += $product_info['model']['support_volume'] * $material['price'.$material_price_fields[$p]];
			}
			elseif ( $material['price_type'.$material_price_fields[$p]]=='support_material_fixed' && $product_info['model']['support_volume']>0) {
				$material_cost += $material['price'.$material_price_fields[$p]];
			}
			elseif ( $material['price_type'.$material_price_fields[$p]]=='removed_material_volume' ) {
				$material_cost += $removed_material_volume * $material['price'.$material_price_fields[$p]];
			}
			elseif ( $material['price_type'.$material_price_fields[$p]]=='total_path' ) {
				$material_cost += (float)$product_info['model']['cut_path'] * $material['price'.$material_price_fields[$p]];
			}
			elseif ( $material['price_type'.$material_price_fields[$p]]=='gram' ) {
				$material_cost += $weight * $material['price'.$material_price_fields[$p]];
			}
			elseif ( $material['price_type'.$material_price_fields[$p]]=='volumetric_weight' ) {
				if ($material['volumetric_weight_indicator']!=0) {
					$volumetric_weight = $product_info['model']['box_volume'] / $material['volumetric_weight_indicator'];
					$max_weight = max($volumetric_weight, $weight);
					$material_cost += $max_weight * $material['price'.$material_price_fields[$p]];
				}
			}
			elseif ( $material['price_type'.$material_price_fields[$p]]=='hour' ) {
				$material_cost += ($product_info['model']['print_time'] /3600) * $material['price'.$material_price_fields[$p]];
			}
			elseif ( $material['price_type'.$material_price_fields[$p]]=='fixed' ) {
				$material_cost += $material['price'.$material_price_fields[$p]];
			}
			elseif ( $material['price_type'.$material_price_fields[$p]]=='pct' ) {
				$material_pct += $material['price'.$material_price_fields[$p]];
			}
		}

		elseif ( strstr ($material['price'.$material_price_fields[$p]], ':' ) ) {
			$material['price'.$material_price_fields[$p]]=trim($material['price'.$material_price_fields[$p]]);
			$material_volume_pricing_array = explode(';', $material['price'.$material_price_fields[$p]]);
			for ($i=count($material_volume_pricing_array)-1; $i>=0; $i--) {
				$discount_rule = $material_volume_pricing_array[$i];
				list ($amount, $price) = explode(':', $discount_rule);
	        		if ( $material['price_type'.$material_price_fields[$p]]=='cm3' ) {
					if ($printing_volume >= $amount) {
						$material_cost += ( $printing_volume ) * $price;
						break;
					}
				}
				elseif ($material['price_type'.$material_price_fields[$p]]=='cm2') {
					if ($product_info['model']['surface_area'] >= $amount) {
						$material_cost += $product_info['model']['surface_area'] * $price;
						break;
					}
				}
	        		elseif ( $material['price_type'.$material_price_fields[$p]]=='removed_material_volume' ) {
					if ($removed_material_volume >= $amount) {
						$material_cost += ( $removed_material_volume ) * $price;
						break;
					}
				}
	        		elseif ( $material['price_type'.$material_price_fields[$p]]=='support_material_volume' ) {
					if ($product_info['model']['support_volume'] >= $amount) {
						$material_cost += $product_info['model']['support_volume'] * $price;
						break;
					}
				}
	        		elseif ( $material['price_type'.$material_price_fields[$p]]=='total_path' ) {
					if ((float)$product_info['model']['cut_path'] >= $amount) {
						$material_cost += (float)$product_info['model']['cut_path'] * $price;
						break;
					}
				}
				elseif ( $material['price_type'.$material_price_fields[$p]]=='gram' ) {
					if ($weight >= $amount) {
						$material_cost += $weight * $price;
						break;
					}
				}
				elseif ( $material['price_type'.$material_price_fields[$p]]=='fixed' ) {
					if ($printing_volume >= $amount) {
						$material_cost += $price;
						break;
					}
				}
			}
		}

	}

	if (is_numeric( $material['min_price'] )) {
		$material_min_price = (float)$material['min_price'];
		if ($material_cost < $material_min_price) $material_cost = $material_min_price;
	}



	for ($p=0;$p<count($printer_price_fields);$p++) {
		if ( is_numeric( $printer['price'.$printer_price_fields[$p]] ) ) {
			if ( $printer['price_type'.$printer_price_fields[$p]]=="material_volume" ) {
				$printing_cost += ( $printing_volume ) * $printer['price'.$printer_price_fields[$p]];

			}
			elseif ( $printer['price_type'.$printer_price_fields[$p]]=="support_material_volume" ) {
				$printing_cost += $product_info['model']['support_volume'] * $printer['price'.$printer_price_fields[$p]];

			}
			elseif ( $printer['price_type'.$printer_price_fields[$p]]=="support_material_fixed" && $product_info['model']['support_volume']>0) {
				$printing_cost += $printer['price'.$printer_price_fields[$p]];

			}
			elseif ( $printer['price_type'.$printer_price_fields[$p]]=="removed_material_volume" ) {
				$printing_cost += $removed_material_volume * $printer['price'.$printer_price_fields[$p]];

			}
			elseif ( $printer['price_type'.$printer_price_fields[$p]]=="total_path" ) {
				$printing_cost += (float)$product_info['model']['cut_path'] * $printer['price'.$printer_price_fields[$p]];
			}
			elseif ( $printer['price_type'.$printer_price_fields[$p]]=="box_volume" ) {
				$printing_cost += $product_info['model']['box_volume'] * $printer['price'.$printer_price_fields[$p]];

			}
			elseif ( $printer['price_type'.$printer_price_fields[$p]]=="box_xy" ) {
				$printing_cost += ($product_info['model']['x_dim'] * $product_info['model']['y_dim']) * $printer['price'.$printer_price_fields[$p]];

			}
			elseif ( $printer['price_type'.$printer_price_fields[$p]]=="gram" ) {
				$printing_cost += $weight * $printer['price'.$printer_price_fields[$p]];

			}
			elseif ( $printer['price_type'.$printer_price_fields[$p]]=="hour" ) {
				$printing_cost += ($product_info['model']['print_time'] /3600) * $printer['price'.$printer_price_fields[$p]];

			}
			elseif ( $printer['price_type'.$printer_price_fields[$p]]=="fixed" ) {
				$printing_cost += $printer['price'.$printer_price_fields[$p]];

			}
			elseif ( $printer['price_type'.$printer_price_fields[$p]]=="pct" ) {
				$printing_pct += $printer['price'.$printer_price_fields[$p]];

			}
		}
		elseif ( strstr ( $printer['price'.$printer_price_fields[$p]], ':' ) ) {
			$printer['price'.$printer_price_fields[$p]]=trim($printer['price'.$printer_price_fields[$p]]);
			$printer_volume_pricing_array = explode(';', $printer['price'.$printer_price_fields[$p]]);

			for ($i=count($printer_volume_pricing_array)-1; $i>=0; $i--) {
				$discount_rule = $printer_volume_pricing_array[$i];
				list ($amount, $price) = explode(':', $discount_rule);
				if ( $printer['price_type'.$printer_price_fields[$p]]=="material_volume" ) {
					if ($printing_volume >= $amount) {
						$printing_cost += ( $printing_volume ) * $price;
						break;
					}
				}
				elseif ( $printer['price_type'.$printer_price_fields[$p]]=="box_volume" ) {
					if ($product_info['model']['box_volume'] >= $amount) {
						$printing_cost += $product_info['model']['box_volume'] * $price;
						break;
					}
				}
				elseif ( $printer['price_type'.$printer_price_fields[$p]]=="removed_material_volume" ) {
					if ($removed_material_volume >= $amount) {
						$printing_cost += $removed_material_volume * $price;
						break;
					}
				}
	        		elseif ( $printer['price_type'.$printer_price_fields[$p]]=='support_material_volume' ) {
					if ($product_info['model']['support_volume'] >= $amount) {
						$printing_cost += $product_info['model']['support_volume'] * $price;
						break;
					}
				}
				elseif ( $printer['price_type'.$printer_price_fields[$p]]=="total_path" ) {
					if ((float)$product_info['model']['cut_path'] >= $amount) {
						$printing_cost += (float)$product_info['model']['cut_path'] * $price;
						break;
					}
				}
				elseif ( $printer['price_type'.$printer_price_fields[$p]]=="gram" ) {
					if ($weight >= $amount) {				
						$printing_cost += $weight * $price;
						break;
					}
				}
				elseif ( $printer['price_type'.$printer_price_fields[$p]]=="fixed" ) {
					if ($printing_volume >= $amount) {
						$printing_cost += $price;
						break;
					}
				}
			}
		}
	}


	if ($product_info['model']['print_time']>0) {
		$print_hours = $product_info['model']['print_time']/3600;
		$printer_energy_hourly_cost = $printer['printer_energy_hourly_cost'];
		$printer_depreciation_hourly_cost = $printer['printer_depreciation_hourly_cost'];
		$printer_repair_hourly_cost = $printer['printer_repair_hourly_cost'];

		$printing_cost += $print_hours * $printer_energy_hourly_cost;
		$printing_cost += $print_hours * $printer_depreciation_hourly_cost;
		$printing_cost += $print_hours * $printer_repair_hourly_cost;
	}


	if (is_numeric( $printer['min_price'] )) {
		$printer_min_price = (float)$printer['min_price'];
		if ($printing_cost < $printer_min_price) $printing_cost = $printer_min_price;
	}

	for ($p=0;$p<count($coating_price_fields);$p++) {
		if ( !empty ( $coating_id ) ) {
			if ( is_numeric($coating['price'.$coating_price_fields[$p]]) ) {
				if ($coating['price_type'.$coating_price_fields[$p]]=='cm2') {
					$coating_cost += $product_info['model']['surface_area'] * $coating['price'.$coating_price_fields[$p]];
				}
				elseif ($coating['price_type'.$coating_price_fields[$p]]=='fixed') {
					$coating_cost += $coating['price'.$coating_price_fields[$p]];
				}
				elseif ($coating['price_type'.$coating_price_fields[$p]]=='pct') {
					$coating_pct += $coating['price'.$coating_price_fields[$p]];
				}
			}

			elseif ( strstr ( $coating['price'.$coating_price_fields[$p]], ':' ) ) {
				$coating['price'.$coating_price_fields[$p]]=trim($coating['price'.$coating_price_fields[$p]]);
				$surface_area_pricing_array = explode(';', $coating['price'.$coating_price_fields[$p]]);

				for ($i=count($surface_area_pricing_array)-1; $i>=0; $i--) {
					$discount_rule = $surface_area_pricing_array[$i];
					list ($amount, $price) = explode(':', $discount_rule);
					if ($coating['price_type'.$coating_price_fields[$p]]=='cm2') {
						if ($product_info['model']['surface_area'] >= $amount) {
							$coating_cost += $product_info['model']['surface_area'] * $price;
							break;
						}
					}
					else if ($coating['price_type'.$coating_price_fields[$p]]=='fixed') {
						if ($product_info['model']['surface_area'] >= $amount) {
							$coating_cost += $price;
							break;
						}
					}
				}
			}
		}
	}


	if (isset($coating) && is_numeric( $coating['min_price'])) {
		$coating_min_price = (float)$coating['min_price'];
		if ($coating_cost < $coating_min_price) $coating_cost = $coating_min_price;
	}
	foreach ($postprocessings_ids as $postprocessing_id) {
		$postprocessing = $postprocessings_array[$postprocessing_id];
		for ($p=0;$p<count($postprocessing_price_fields);$p++) {

			if ( !empty ( $postprocessing_id ) ) {
				if ( is_numeric($postprocessing['price'.$postprocessing_price_fields[$p]]) ) {
					if ((int)$postprocessing['is_support']==1 && (float)$product_info['model']['support_volume']==0) continue;

					if ($postprocessing['price_type'.$postprocessing_price_fields[$p]]=='cm2') {
						$postprocessing_cost += $product_info['model']['surface_area'] * $postprocessing['price'.$postprocessing_price_fields[$p]];
					}
					else if ( $postprocessing['price_type'.$postprocessing_price_fields[$p]]=="material_volume" ) {
						$postprocessing_cost += ( $printing_volume ) * $postprocessing['price'.$postprocessing_price_fields[$p]];

					}
					elseif ( $postprocessing['price_type'.$postprocessing_price_fields[$p]]=="support_material_volume" ) {
						$postprocessing_cost += $product_info['model']['support_volume'] * $postprocessing['price'.$postprocessing_price_fields[$p]];

					}
					elseif ( $postprocessing['price_type'.$postprocessing_price_fields[$p]]=="support_material_fixed" && $product_info['model']['support_volume']>0) {
						$postprocessing_cost += $postprocessing['price'.$postprocessing_price_fields[$p]];

					}
					elseif ( $postprocessing['price_type'.$postprocessing_price_fields[$p]]=="removed_material_volume" ) {
						$postprocessing_cost += $removed_material_volume * $postprocessing['price'.$postprocessing_price_fields[$p]];

					}
					elseif ( $postprocessing['price_type'.$postprocessing_price_fields[$p]]=="total_path" ) {
						$postprocessing_cost += (float)$product_info['model']['cut_path'] * $postprocessing['price'.$postprocessing_price_fields[$p]];
					}
					elseif ( $postprocessing['price_type'.$postprocessing_price_fields[$p]]=="box_volume" ) {
						$postprocessing_cost += $product_info['model']['box_volume'] * $postprocessing['price'.$postprocessing_price_fields[$p]];

					}
					elseif ( $postprocessing['price_type'.$postprocessing_price_fields[$p]]=="box_xy" ) {
						$postprocessing_cost += ($product_info['model']['x_dim'] * $product_info['model']['y_dim']) * $postprocessing['price'.$postprocessing_price_fields[$p]];

					}
					elseif ( $postprocessing['price_type'.$postprocessing_price_fields[$p]]=="gram" ) {
						$postprocessing_cost += $weight * $postprocessing['price'.$postprocessing_price_fields[$p]];

					}
					elseif ( $postprocessing['price_type'.$postprocessing_price_fields[$p]]=="hour" ) {
						$postprocessing_cost += ($product_info['model']['print_time'] /3600) * $postprocessing['price'.$postprocessing_price_fields[$p]];

					}
					elseif ($postprocessing['price_type'.$postprocessing_price_fields[$p]]=='fixed') {
						$postprocessing_cost += $postprocessing['price'.$postprocessing_price_fields[$p]];
					}
					elseif ($postprocessing['price_type'.$postprocessing_price_fields[$p]]=='pct') {
						$postprocessing_pct += $postprocessing['price'.$postprocessing_price_fields[$p]];
					}
				}

				elseif ( strstr ( $postprocessing['price'.$postprocessing_price_fields[$p]], ':' ) ) {
					$postprocessing['price'.$postprocessing_price_fields[$p]]=trim($postprocessing['price'.$postprocessing_price_fields[$p]]);
					$surface_area_pricing_array = explode(';', $postprocessing['price'.$postprocessing_price_fields[$p]]);
	
					for ($i=count($surface_area_pricing_array)-1; $i>=0; $i--) {
						$discount_rule = $surface_area_pricing_array[$i];
						list ($amount, $price) = explode(':', $discount_rule);
						if ($postprocessing['price_type'.$postprocessing_price_fields[$p]]=='cm2') {
							if ($product_info['model']['surface_area'] >= $amount) {
								$postprocessing_cost += $product_info['model']['surface_area'] * $price;
								break;
							}
						}
						if ( $postprocessing['price_type'.$postprocessing_price_fields[$p]]=="material_volume" ) {
							if ($printing_volume >= $amount) {
								$postprocessing_cost += ( $printing_volume ) * $price;
								break;
							}
						}
						elseif ( $postprocessing['price_type'.$postprocessing_price_fields[$p]]=="box_volume" ) {
							if ($product_info['model']['box_volume'] >= $amount) {
								$postprocessing_cost += $product_info['model']['box_volume'] * $price;
								break;
							}
						}
						elseif ( $postprocessing['price_type'.$postprocessing_price_fields[$p]]=="removed_material_volume" ) {
							if ($removed_material_volume >= $amount) {
								$postprocessing_cost += $removed_material_volume * $price;
								break;
							}
						}
			        		elseif ( $postprocessing['price_type'.$postprocessing_price_fields[$p]]=='support_material_volume' ) {
							if ($product_info['model']['support_volume'] >= $amount) {
								$postprocessing_cost += $product_info['model']['support_volume'] * $price;
								break;
							}
						}
						elseif ( $postprocessing['price_type'.$postprocessing_price_fields[$p]]=="total_path" ) {
							if ((float)$product_info['model']['cut_path'] >= $amount) {
								$postprocessing_cost += (float)$product_info['model']['cut_path'] * $price;
								break;
							}
						}
						else if ($postprocessing['price_type'.$postprocessing_price_fields[$p]]=='fixed') {
							if ($product_info['model']['surface_area'] >= $amount) {
								$postprocessing_cost += $price;
								break;
							}
						}
					}
				}
			}
		}

#var_dump($postprocessing_cost);
		if (isset($postprocessing) && is_numeric( $postprocessing['min_price'])) {
			$postprocessing_min_price = (float)$postprocessing['min_price'];
			if ($postprocessing_cost < $postprocessing_min_price) $postprocessing_cost += $postprocessing_min_price;
		}
	}

	for ($p=0;$p<count($infill_price_fields);$p++) {
		if ( isset($infill['price'.$infill_price_fields[$p]]) && is_numeric($infill['price'.$infill_price_fields[$p]]) ) {
			if ($infill['price_type'.$infill_price_fields[$p]]=='fixed') {
				$infill_cost += $infill['price'.$infill_price_fields[$p]];
			}
			elseif ($infill['price_type'.$infill_price_fields[$p]]=='pct') {
				$infill_pct += $infill['price'.$infill_price_fields[$p]];
			}
			elseif ($infill['price_type'.$infill_price_fields[$p]]=='pct_mat') {
				$material_cost += ($material_cost/100) * (float)$infill['price'.$infill_price_fields[$p]];
			}
		}
	}



	if (count($attributes)) {
		foreach ($attributes as $attr_name => $attr_value) {
			$attr_name=str_replace('attribute_', '', $attr_name);
			if (isset($p3d_attr_prices[$attr_name][$attr_value])) {
				$attr_price=(float)$p3d_attr_prices[$attr_name][$attr_value]['price'];
				$attr_price_type=$p3d_attr_prices[$attr_name][$attr_value]['price_type'];
				$attr_pct_type=$p3d_attr_prices[$attr_name][$attr_value]['pct_type'];
				if ($attr_price_type=='pct') {
					if ($attr_pct_type=='printer') {
						$printing_cost+=($printing_cost/100)*$attr_price;
					}
					if ($attr_pct_type=='material') {
						$material_cost+=($material_cost/100)*$attr_price;
					}
					if ($attr_pct_type=='coating') {
						$coating_cost+=($coating_cost/100)*$attr_price;
					}
				}
			}
		}
	}
#echo "$printing_cost+$material_cost+$coating_cost+$infill_cost <br>";
#var_dump($postprocessing_cost);
	$total = $printing_cost+$material_cost+$coating_cost+$infill_cost+$postprocessing_cost;

	$total += ($total/100)*($printing_pct+$material_pct+$coating_pct+$infill_pct+$postprocessing_pct);



	$attr_total = 0;
	if (count($attributes)) {
		foreach ($attributes as $attr_name => $attr_value) {
			$attr_name=str_replace('attribute_', '', $attr_name);
			if (isset($p3d_attr_prices[$attr_name][$attr_value])) {
				$attr_price=(float)$p3d_attr_prices[$attr_name][$attr_value]['price'];
				$attr_price_type=$p3d_attr_prices[$attr_name][$attr_value]['price_type'];
				if (isset($p3d_attr_prices[$attr_name][$attr_value]['flat_type'])) {
					$attr_flat_type=$p3d_attr_prices[$attr_name][$attr_value]['flat_type'];
				}
				else {
					$attr_flat_type='';
				}
				$attr_pct_type=$p3d_attr_prices[$attr_name][$attr_value]['pct_type'];

				if ($attr_price_type=='flat') {
					if ($attr_flat_type=='support_material_amount' && $product_info['model']['support_volume']>0) {
						$support_price = $attr_price*$product_info['model']['support_volume'];
						$attr_total+=$support_price;
						$total+=$support_price;
					}
					elseif ($attr_flat_type=='support_material_flag' && $product_info['model']['support_volume']>0) {
						$support_price = $attr_price;
						$attr_total+=$support_price;
						$total+=$support_price;
					}
					else {
						$total+=$attr_price;
						$attr_total+=$attr_price;
					}
				}
				else if ($attr_price_type=='pct') {
					if ($attr_pct_type=='total') { 
						$total+=($total/100)*$attr_price;
						$attr_total+=($total/100)*$attr_price;
					}
				}
			}
		}
	}

	if ($settings['minimum_price_type']=='starting_price') {
		$total = $total + $min_price;
	}
	elseif ($settings['minimum_price_type']=='minimum_price') {
		if ($total < $min_price) 
			$total = $min_price + $attr_total;
	}


	if (strlen($p3d_file_url)>0 && $p3d_product_price_type=='fixed') {
		$total = $min_price + $attr_total;
	}


	//DO SOMETHING WITH THE TOTAL HERE
	return $total;
}



?>